#version 130

in  vec2 ex_texCoord;
out vec4 out_Color;

uniform bool blur_direction;
uniform sampler2D un_texture;
uniform sampler2D depth_texture;
uniform float focus;

float convertDepth(float depth) {
	return (depth - focus) * 1.0/(1.0 - focus);
}

void main(void)
{
	float depth = texture2D(depth_texture, ex_texCoord).x;
	
	float depth_border = 0.00; //0.95
	
	//convert depth (most of the scene is at depth > 0.9, <0.9 is about 5-10 units from camera)
	depth = convertDepth(depth);
	
	if (depth < depth_border) {
		out_Color = texture2D(un_texture, ex_texCoord);
	} else {
		//float min_dist = 0.002;
		vec2 blur_dist = vec2(0.002 * depth, 0.003 * depth); //at default resolution, more then 0.001 shows visible 3 overlapped images
		
		//mean
		//vec3 weight = vec3(0.2, 0.2, 0.2);
		//test
		//vec3 weight = vec3(0.3, 0.2, 0.15);
		vec4 weight = vec4(0.25, 0.2, 0.125, 0.1);
		
		if (blur_direction) { //x-direction
			vec4 center_color = texture2D(un_texture, ex_texCoord+vec2(0.000, 0.000));
			out_Color =	center_color*weight.x;
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(blur_dist.x, 0.000)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(blur_dist.x, 0.000))*weight.y; //default
			} else {
				out_Color += center_color*weight.y; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(-blur_dist.x, 0.000)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(-blur_dist.x, 0.000))*weight.y; //default
			} else {
				out_Color += center_color*weight.y; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(2*blur_dist.x, 0.000)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(2*blur_dist.x, 0.000))*weight.z; //default
			} else {
				out_Color += center_color*weight.z; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(-2*blur_dist.x, 0.000)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(-2*blur_dist.x, 0.000))*weight.z; //default
			} else {
				out_Color += center_color*weight.z; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(-2*blur_dist.x, 0.000)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(-2*blur_dist.x, 0.000))*weight.w; //default
			} else {
				out_Color += center_color*weight.w; //no blur if not in blur distance
			}
			
			
		} else { //y-direction
			vec4 center_color = texture2D(un_texture, ex_texCoord+vec2(0.000, 0.000));
			out_Color = center_color*weight.x;
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(0.000, blur_dist.y)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(0.000, blur_dist.y))*weight.y; //default
			} else {
				out_Color += center_color*weight.y; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(0.000, -blur_dist.y)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(0.000, -blur_dist.y))*weight.y; //default
			} else {
				out_Color += center_color*weight.y; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(0.000, 2*blur_dist.y)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(0.000, 2*blur_dist.y))*weight.z; //default
			} else {
				out_Color += center_color*weight.z; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(0.000, -2*blur_dist.y)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(0.000, -2*blur_dist.y))*weight.z; //default
			} else {
				out_Color += center_color*weight.z; //no blur if not in blur distance
			}
			
			//additional lookup against blurred edges
			if (convertDepth(texture2D(depth_texture, ex_texCoord+vec2(0.000, -2*blur_dist.y)).x) > depth_border) {
				out_Color += texture2D(un_texture, ex_texCoord+vec2(0.000, -2*blur_dist.y))*weight.w; //default
			} else {
				out_Color += center_color*weight.w; //no blur if not in blur distance
			}
			
		}
		
	}
	
	//fog
	if (depth > depth_border) {
		//out_Color*=(1.0-depth/10.0);
	}
	
}
