// Vertex Shader â€“ file "minimal.vert"

#version 130

//IN VERTEX ATTRIBUTES
in  vec3 in_Position;
in	vec2 in_texCoord;
in	vec3 in_Normal;

//OUT VERTEX ATTRIBUTES
out vec3 ex_Intensity;
out vec2 ex_texCoord;
out vec4 ex_shadow_tex_coord;

//UNIFORM OBJECT TRANSFORMATION
uniform vec3 in_Object_Translation;
uniform float rot_Object_angle_X;
uniform float rot_Object_angle_Y;
uniform float rot_Object_angle_Z;
uniform float in_Object_Scale;

//UNIFORM TRANSFORMATION PASSING
uniform vec3 in_lightPos;
uniform vec3 in_Translation;
uniform vec3 in_Scale;
uniform float rot_angle_X;
uniform float rot_angle_Y;

//UNIFORM SHADOW (LIGHT) TRANSFORMATION
uniform vec3 in_Shadow_Translation;
uniform float shadow_rot_angle_X;
uniform float shadow_rot_angle_Y;

//UNIFORM MATRIX PASSING - NOT USED AT THE MOMENT
//uniform mat4 modelview_matrix;
//uniform mat4 projection_matrix;

//LOCAL LIGHT VARIABLES

//mobile lightsource
vec3 lightPos = vec3(20.0,5.0,0.0); //DIRECTION!?!10.0,7.0,-10.0
const float specContr = 0.3;
const float diffContr = 2.0 - specContr;//
const vec3 lightDirection = vec3(0.0,-1.0,0.0);//downward
const float cosInnerCone = 0.866;//30 deg
const float cosOuterCone = 0.5;//60 deg
const float attenC = 1.0;//constant attenuation
const float attenL = 2.0;//linear attenuation
const float attenQ = 0.0;//quadratic attenuation

//fixed light - skylight
const vec3 lightPosG = vec3(1700.0,1700.0,1700.0);
const float specContrG = 0.3;
const float diffContrG = 1.0 - specContr;

//LOCAL TRANSFORMATION VARIABLES

const int MODELVIEW = 0;
const int PROJECTION = 1;

int current_mode = MODELVIEW;
mat4 current_matrix = mat4(1.0);
mat4 MV_matrix = mat4(1.0);
mat4 PROJ_matrix = mat4(1.0);
mat3 NORMAL_matrix = mat3(1.0);
mat4 light_matrix = mat4(1.0);

//////////////////////////////////////
//PROJECTION MATRIX CALCULATOR		//
//////////////////////////////////////
mat4 aplProjectionMatrix(const in float left, const in float right,
                         const in float bottom, const in float top,
                         const in float zNear, const in float zFar){

	vec3 delta	= vec3(right, top, zFar) - vec3(left, bottom, zNear);
	vec3 sum	= vec3(right, top, zFar) + vec3(left, bottom, zNear);
	vec3 ratio	= sum / delta;
	vec3 twoRatio = 2.0 * zNear / delta;
	return mat4(twoRatio.x,		0.0,			0.0,			0.0, 
				0.0,			twoRatio.y,		0.0,			0.0, 
				ratio.x,		ratio.y,		-ratio.z,		-1.0,
				0.0,			0.0,			-zNear*twoRatio.z,		0.0);
}
//////////////////////////////////////
//TRANSFORMATION MATRIX CALCULATOR	//
//////////////////////////////////////
mat4 aplIdentityMatrix(){
	return mat4(1.0, 0.0, 0.0, 0.0,
				0.0, 1.0, 0.0, 0.0,
				0.0, 0.0, 1.0, 0.0,
				0.0, 0.0, 0.0, 1.0);
}

mat4 aplTranslationsMatrix(const in vec3 translate){
	return mat4(1.0,		0.0,		0.0,			0.0, 
				0.0,		1.0,		0.0,			0.0, 
				0.0,		0.0,		1.0,			0.0,
				translate.x, translate.y, translate.z, 1.0);
}

mat4 aplScaleMatrix(const in vec3 scale){
	return mat4(scale.x,	0.0,		0.0,		0.0,
				0.0,		scale.y,	0.0,		0.0,
				0.0,		0.0,		scale.z,	 0.0,
				0.0,		0.0,		0.0,		1.0);
	
}

mat4 aplRotateMatrix(const in float angle, const in vec3 axis){
	vec3 n = normalize(axis);
	float theta = radians(angle);
	float c = cos(theta);
	float s = sin(theta);
	mat3 R;
	R[0] = n.xyz*n.x*(1.0-c) + vec3( c,		 n.z*s,	-n.y*s);
	R[1] = n.xyz*n.y*(1.0-c) + vec3(-n.z*s,	 c,		 n.x*s);
	R[2] = n.xyz*n.z*(1.0-c) + vec3( n.y*s,	-n.x*s,		 c);
	return mat4(R[0],			0.0,
				R[1],			0.0,
				R[2],			0.0,
				0.0, 0.0, 0.0,  1.0);

}

mat3 aplRotateVecMatrix(const in float angle, const in vec3 axis){
	vec3 n = normalize(axis);
	float theta = radians(angle);
	float c = cos(theta);
	float s = sin(theta);
	mat3 R;
	R[0] = n.xyz*n.x*(1.0-c) + vec3( c,		 n.z*s,	-n.y*s);
	R[1] = n.xyz*n.y*(1.0-c) + vec3(-n.z*s,	 c,		 n.x*s);
	R[2] = n.xyz*n.z*(1.0-c) + vec3( n.y*s,	-n.x*s,		 c);
	return R;

}

mat4 aplRotateMatrixTwoAxes(const in float angle1, const in vec3 axis1,
							const in float angle2, const in vec3 axis2){
	return aplRotateMatrix(angle1, axis1)*aplRotateMatrix(angle2, axis2);
}

//////////////////////////////////////////////////////////////////
//MULTIPLYING TRANSFORMATION  AND PROJECTION WITH CURRENT MATRIX//
//////////////////////////////////////////////////////////////////
void aplMatrixMode(const in int mode){
	if (mode != current_mode){
		switch (mode){
			case MODELVIEW:
			{
				PROJ_matrix = current_matrix;
				current_matrix = MV_matrix;
				current_mode = MODELVIEW;
				break;
			}
			case PROJECTION:
			{
				MV_matrix = current_matrix;
				current_matrix = PROJ_matrix;
				current_mode = PROJECTION;
				break;
			}
			default:
			{
				break;
			}
		}
	}
	return;
}

void aplLoadIdentity(void){
	current_matrix = aplIdentityMatrix();
}

void aplLoadMatrix(const in mat4 m){
	current_matrix = m;
}

void aplMultMatrix(const in mat4 m){
	current_matrix = m * current_matrix;
}

void aplScale(const in vec3 s){
	current_matrix = aplScaleMatrix(s)*current_matrix;
}

void aplTranslate(const in vec3 t){
	current_matrix = aplTranslationsMatrix(t)*current_matrix;
}

void aplRotate(const in float angle, const in vec3 axis){
	current_matrix = aplRotateMatrix(angle, axis)*current_matrix;
}

void aplFrustum(const in float left, const in float right,
                const in float bottom, const in float top,
                const in float zNear, const in float zFar){
	               
	current_matrix = aplProjectionMatrix(left, right, bottom, top,
										 zNear, zFar)*current_matrix;        
 }

//////////////////////////////////////
//LIGHT PROPERTIES CALCULATOR       //
//////////////////////////////////////
float computeSpotlight(vec3 viewVect, vec3 lightDir, float cosInner, float cosOuter){

    //assumes vewVect is normalized
	float cosDir = dot(viewVect,lightDir);
	return smoothstep(cosOuter, cosInner, cosDir);
}

float computeAttenuation(vec3 pos, vec3 _lightPos, float kC, float kL, float kQ){
	
	float d = distance(pos, _lightPos);
	return 1/(kC + kL*d + kQ*d*d);
}


//////////////////////////////////////
//MAIN								//
//////////////////////////////////////
void main(void)
{	
	aplMatrixMode(PROJECTION);
	aplLoadIdentity();
	aplFrustum(-8, 8, -6, 6, 20, 1500); //dependant on window //20, 500
	
	aplMatrixMode(MODELVIEW);//swaps matrices!
	aplLoadIdentity();
	
	aplScale(vec3(in_Object_Scale, in_Object_Scale, in_Object_Scale));
	aplRotate(rot_Object_angle_X, vec3(1.0, 0.0, 0.0));		//object rotation
	aplRotate(rot_Object_angle_Y, vec3(0.0, 1.0, 0.0));		//object rotation
	aplRotate(rot_Object_angle_Z, vec3(0.0, 0.0, 1.0));		//object rotation
	aplTranslate(in_Object_Translation);					//object location
	
	aplScale(in_Scale);
	aplRotate(rot_angle_Y, vec3(0.0, 1.0, 0.0));
	aplRotate(rot_angle_X, vec3(1.0, 0.0, 0.0));
	aplTranslate(in_Translation);//in_Translation
	//test
	//aplRotate(-shadow_rot_angle_Y, vec3(0.0, 1.0, 0.0));
	//aplRotate(-shadow_rot_angle_X, vec3(1.0, 0.0, 0.0));
	//aplTranslate(-1.0*in_Shadow_Translation);
	aplMatrixMode(PROJECTION);//swaps matrices!
	
	//################################################## START LIGHT MATRIX ##################
	
	//LIGHT MODELVIEW
	light_matrix = aplIdentityMatrix();
	
	light_matrix = aplScaleMatrix(vec3(in_Object_Scale, in_Object_Scale, in_Object_Scale))*light_matrix;
	light_matrix = aplRotateMatrix(rot_Object_angle_X, vec3(1.0, 0.0, 0.0))*light_matrix;
	light_matrix = aplRotateMatrix(rot_Object_angle_Y, vec3(0.0, 1.0, 0.0))*light_matrix;
	light_matrix = aplRotateMatrix(rot_Object_angle_Z, vec3(0.0, 0.0, 1.0))*light_matrix;
	light_matrix = aplTranslationsMatrix(in_Object_Translation)*light_matrix;
	
	light_matrix = aplScaleMatrix(in_Scale)*light_matrix;
	light_matrix = aplRotateMatrix(shadow_rot_angle_Y, vec3(0.0, 1.0, 0.0))*light_matrix;
	light_matrix = aplRotateMatrix(shadow_rot_angle_X, vec3(1.0, 0.0, 0.0))*light_matrix;
	light_matrix = aplTranslationsMatrix(vec3(in_Shadow_Translation.x, in_Shadow_Translation.y, -in_Shadow_Translation.z))*light_matrix;
	//LIGHT PROJECTION
	light_matrix = PROJ_matrix*light_matrix;
	//LIGHT SCALE
	light_matrix = aplScaleMatrix(vec3(0.5, 0.5, 0.5))*light_matrix;
	//LIGHT TRANSLATE
	light_matrix = aplTranslationsMatrix(vec3(0.5, 0.5, 0.5))*light_matrix;
	
	//#################################################### END LIGHT MATRIX ##################
	
	vec3 ec_Position = vec3(MV_matrix * vec4(in_Position,1.0));
	vec4 ecv_Position = vec4(ec_Position, 1.0);
	gl_Position = vec4(PROJ_matrix * ecv_Position);

	//NORMAL MATRIX
	mat3 curVecMatr = mat3(1.0);
	curVecMatr = aplRotateVecMatrix(rot_angle_Y, vec3(0.0, 1.0, 0.0))*curVecMatr;
	NORMAL_matrix = aplRotateVecMatrix(rot_angle_X, vec3(1.0, 0.0, 0.0))*curVecMatr;
	
	{
		//LIGHT INTENSITY CALCULATION
	
		vec3 normVec = normalize(NORMAL_matrix*in_Normal);
		vec3 viewVec = normalize(ec_Position);
		vec3 ec_lightPos = vec3(MV_matrix*vec4(in_lightPos,1.0));//vec3 ec_lightPos = NORMAL_matrix*lightPos;
		vec3 ec_lightDirection = NORMAL_matrix*lightDirection;
		vec3 ec_lightPosG = vec3(MV_matrix*vec4(lightPosG,1.0));//vec3 ec_lightPosG = NORMAL_matrix*lightPosG;
		
		//mobile light source (ORANGE)
		
		vec3 lightVec = normalize(ec_lightPos - ec_Position);
		vec3 reflVec = reflect(-lightVec,normVec);
		reflVec = normalize((PROJ_matrix*vec4(reflVec, 0.0)).xyz);
		
		float diffuse = max(dot(lightVec,normVec),0.0);
		float spec = 0.0;
		if (diffuse > 0.0){
			spec = max(dot(reflVec,viewVec),0.0);
			spec = pow(spec, 16.0);//sharpen specular part
		}
		float spotEffect = computeSpotlight(viewVec, ec_lightDirection, cosInnerCone, cosOuterCone);
		float attenEffect = computeAttenuation(ec_Position, ec_lightPos, attenC, attenL, attenQ);
		diffuse *= attenEffect;//*spotEffect;
		spec *= attenEffect;//*spotEffect;
		
		
		//fixed light source (WHITE)
		
		vec3 lightVecG = normalize(ec_lightPosG - ec_Position);
		vec3 reflVecG = reflect(-lightVecG,normVec);
		reflVecG = normalize((PROJ_matrix*vec4(reflVecG, 0.0)).xyz);
		
		float diffuseG = max(dot(lightVecG,normVec),0.0);
		float specG = 0.0;
		if (diffuseG > 0.0){
			specG = max(dot(reflVecG,viewVec),0.0);
			specG = pow(specG, 32.0);//sharpen specular part
		}
		
		//ambient + light1 + light2
		//0.3 + 0.9 + 0.95; 0.3 + 0.85 + 0.95; 0.3 + 0.7 + 0.7
		float lightIntRED = 0.3 + 1*(diffContr*diffuse + specContr*spec) + 0.95*(diffContrG*diffuseG + specContrG*specG);  //1,0.59/0.7,0.95
		float lightIntGREEN = 0.3 + 0.706*(diffContr*diffuse + specContr*spec) + 0.95*(diffContrG*diffuseG + specContrG*specG);  //0.706,1/0.7,0.95
		float lightIntBLUE = 0.3 + 0.0*(diffContr*diffuse + specContr*spec) + 0.7*(diffContrG*diffuseG + specContrG*specG);//0,0/0.5,0.7
		
		ex_Intensity.r = lightIntRED;//RED
		ex_Intensity.g = lightIntGREEN;//GREEN
		ex_Intensity.b = lightIntBLUE;//BLUE
		
		//if (in_Position.z > 10) { //test: position independant from camera
		//	ex_Intensity.r = 1.0;//RED
		//	ex_Intensity.g = 0;//GREEN
		//	ex_Intensity.b = 0;
		//}
	}

	ex_shadow_tex_coord = light_matrix * vec4(in_Position, 1.0);
	ex_texCoord = in_texCoord;
	//ex_Normal = (MV_matrix*vec4(in_Normal, 0.0)).xyz; //OLD
}